import os

from analyzer.rating import Rating
from analyzer.model import Author
from analyzer.model import Article

def rating():
    user = os.environ.get('PTI_USER')
    password = os.environ.get('PTI_PASSWORD')
    host = os.environ.get('PTI_HOST')
    db = os.environ.get('PTI_DB')
    articles = Article.load_from_mysql(user, password, host, db, '/var/pti/topic')
    r = Rating(articles)
    r.calc()
    authors = Author.load_from_mysql(user, password, host, db)
    for a in authors:
        rate = r.get_author_rate(a.id)
        a.rate = rate
        a.save(user, password, host, db)

if __name__ == "__main__":
    rating()


from datetime import datetime, date, time

from analyzer.model import Article
from analyzer.model import Author
from analyzer.rating import Rating


class TestRating(object):
    articles = [
        Article(0, Author(0, "Kai",       0), datetime(2017, 1, 23), "http://a.com", [(0, 0.1), (1, 0.2), (2, 0.3)]),
        Article(1, Author(1, "Publisher", 0), datetime(2017, 1, 24), "http://b.com", [(0, 0.3), (1, 0.2), (2, 0.1)]),
        Article(2, Author(2, "Gologo",    0), datetime(2017, 1, 25), "http://c.com", [(0, 0.6), (1, 0.3), (2, 0.9)]),
        Article(3, Author(3, "Yamada",    0), datetime(2017, 1, 26), "http://d.com", [(0, 0.6), (1, 0.3), (2, 0.9)]),
        Article(4, Author(2, "Gologo",    0), datetime(2017, 1, 27), "http://e.com", [(0, 0.1), (1, 0.5), (2, 0.1)]),
        Article(5, Author(2, "Gologo",    0), datetime(2017, 1, 28), "http://f.com", [(0, 0.6), (1, 0.2), (2, 0.4)])
    ]

    rating = Rating(articles)

    def test_extract_topics(self):
        topics = TestRating.rating.unique_topics()
        assert len(topics) == 3

    def test_num_unique_topics(self):
        num_topics = TestRating.rating.num_unique_topics()
        assert num_topics == 3

    def test_unique_author(self):
        unique_authors = TestRating.rating.unique_authors()
        assert len(unique_authors) == 4

    def test_rating_calculation(self):
        ret = TestRating.rating.calc()
        assert len(ret) == 4
        assert ret[0] == 0
        assert ret[1] == 0
        assert ret[2] == 0
        assert ret[3] == -0.46211715726000974



import os
import pytest
from datetime import datetime, date, time

from analyzer.model import Article
from analyzer.model import Author


class TestModel:
    def test_author_name(self):
        a = Author(0, "Kai", 3)
        assert a.name == "Kai"

    def test_article_name(self):
        a = Author(0, "Kai", 3)
        d = date(2017, 2, 4)
        t = time(13, 22)
        topics = [(1, 0.32), (2, 0.12), (3, 0.21)]
        article = Article(1, a, datetime.combine(d, t), "http://example.com", topics)
        assert article.author == a
        assert article.pub_time.year == 2017
        assert len(article.topics) == 3

    @pytest.mark.skip(reason="depends on remote MySQL")
    def test_author_db(self):
        user = os.environ.get('PTI_USER')
        password = os.environ.get('PTI_PASSWORD')
        host = os.environ.get('PTI_HOST')
        db = os.environ.get('PTI_DB')
        authors = Author.load_from_mysql(user, password, host, db)
        assert len(authors) == 1

    @pytest.mark.skip(reason="no pickle file in predefined dir")
    def test_article_db(self):
        user = os.environ.get('PTI_USER')
        password = os.environ.get('PTI_PASSWORD')
        host = os.environ.get('PTI_HOST')
        db = os.environ.get('PTI_DB')
        articles = Article.load_from_mysql(user, password, host, db)
        assert len(articles) == 2

    def test_pickle_load(self):
        t = Article.load_from_pickel("./data", 1)
        assert len(t) == 2




# -*- coding: utf-8 -*-

import numpy as np
from analyzer.model import Article


class Rating(object):

    MAX_GAIN = 10.0
    DISCOUNT = 0.5
    RATING_BASE = 1.0
    MAX_ARTICLES = 5
    TOPIC_PROB_THRESHOLD = 0.5
    '''
    Rating calculate rating of each authors given articles
    according topic model.
    '''

    def __init__(self, articles):
        self.articles = articles
        # Rating result is calculated like key-value between author and rate
        # e.g. {author1: 98, author2: 12, author3: 43, ... }
        ratings = self.calc()

    def unique_authors(self):
        return set([a.author for a in self.articles])

    def unique_topics(self):
        '''
        List of unique topics given articles
        :param articles:
        :return: list of unique topic ids
        '''
        x = []
        for a in self.articles:
            x.extend(a.topics)
        unique_topics = set([t[0] for t in x])
        return unique_topics

    def num_unique_topics(self):
        '''
        Number of unique topics
        :param articles:
        :return: number of unique topics
        '''
        return len(self.unique_topics())

    def calc(self):
        '''
        Calculate rating of each authors
        :return: key-value dictionary of author and their rating
        '''
        ret = {a.id: 0 for a in self.unique_authors()}
        unique_topics = self.unique_topics()

        for t in unique_topics:
            # Filter topic related articles
            related_articles = [art for art in self.articles
                                 if art.get_topic_prob(t) >= Rating.TOPIC_PROB_THRESHOLD]

            # Sorted by time in descending order
            # First articles should get lower points because it's published late.
            sorted_by_time = list(reversed(sorted(related_articles,
                                                  key=lambda a: a.pub_time)))

            # Select above 50% articles sorted by topic probability
            num_articles = len(sorted_by_time)

            # Search position of publisher's article
            publisher_index = int(num_articles / 2)
            for i, a in enumerate(sorted_by_time):
                if a.author.id == 1:
                    publisher_index = i
                    break

            for i, s in enumerate(sorted_by_time):
                ret[s.author.id] += np.tanh((i - publisher_index) / num_articles)

        self.rating = ret
        return self.rating

    def get_author_rate(self, author_id):
        return self.rating[author_id]



import sys

from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os
import re

import sys

from setuptools.command.test import test as TestCommand
from tests.pytest_helper import PyTest


def read(*names, **kwargs):
    with open(
            os.path.join(os.path.dirname(__file__), *names),
            encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pti-analyzer',
    version=find_version("analyzer", "__init__.py"),
    description='Analyzing articles based on topic model(LDA)',
    long_description=long_description,
    url='https://github.com/PhysicsEngine/pti-analyzer',
    author='Kai Sasaki',
    author_email='lewuathe@me.com',
    license='Apache V2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    keywords='machine-learning topic-model LDA',
    packages=find_packages(exclude=['tests*']),
    install_requires=['numpy', 'PyMySQL'],
    tests_require=['pytest', 'pytest-cov'],
    cmdclass={'test': PyTest}
)


# -*- coding: utf-8 -*-

import pymysql.cursors
import os


def check_consistency(user, password, host, database):
    conn = pymysql.connect(user=user, password=password,
                           host=host, database=database)

    try:
        with conn.cursor() as cur:
            sql = "SELECT id from articles_articles;"
            cur.execute(sql)
            ids = [r[0] for r in cur.fetchall()]
            for id in ids:
                if not os.path.exists("/var/pti/scrape/{}.txt".format(id)):
                    print("Not found scraped text: id -> {}".format(id))

    finally:
        conn.close()


if __name__ == "__main__":
    user = os.environ.get('PTI_USER')
    password = os.environ.get('PTI_PASSWORD')
    host = os.environ.get('PTI_HOST')
    db = os.environ.get('PTI_DB')
    check_consistency(user, password, host, db)



# -*- coding: utf-8 -*-

import pymysql.cursors
import pickle


class Author(object):
    '''
    Author represents a author of article which keeps
    rating of the author.
    '''

    @classmethod
    def load_from_mysql(cls, user, password, host, database):
        conn = pymysql.connect(user=user, password=password,
                               host=host, database=database)

        try:
            with conn.cursor() as cur:
                sql = "SELECT id,name,rate from articles_authors;"
                cur.execute(sql)
                ret = [Author(r[0], r[1], r[2]) for r in cur.fetchall()]
        finally:
            conn.close()

        return ret

    def save(self, user, password, host, database):
        conn = pymysql.connect(user=user, password=password,
                               host=host, database=database)

        try:
            with conn.cursor() as cur:
                sql = "UPDATE articles_authors " \
                      "  SET rate=%s " \
                      "  WHERE id=%s;"
                cur.execute(sql, (str(self.rate), self.id))
        finally:
            conn.close()

        return

    def __init__(self, id, name, rate):
        self.id = id
        self.name = name
        self.rate = rate

    def __eq__(self, other):
        assert type(other) is Author
        return self.name == other.name and self.id == other.id

    def __ne__(self, other):
        assert type(other) is Author
        return self.name != other.name and self.id == other.id

    def __hash__(self):
        return self.id.__hash__()


class Article(object):
    '''
    Article represents a article by a author.
    '''

    @classmethod
    def load_from_mysql(cls, user, password, host, database, pkl_base_dir):
        conn = pymysql.connect(user=user, password=password,
                               host=host, database=database)

        ret = []
        try:
            with conn.cursor() as cur:
                sql = "select " \
                      "  art.id," \
                      "  art.pub_date," \
                      "  art.url," \
                      "  auh.id," \
                      "  auh.name," \
                      "  auh.rate " \
                      "from " \
                      "  articles_articles art " \
                      "JOIN " \
                      "  articles_authors auh " \
                      "on art.author_id = auh.id;"
                cur.execute(sql)

                for r in cur.fetchall():
                    try:
                        topics = Article.load_from_pickel(pkl_base_dir, r[0])
                        a = Article(r[0], Author(r[3], r[4], r[5]), r[1], r[2], topics)
                        ret.append(a)
                    except FileNotFoundError as e:
                        print("File Not Found", e)

        finally:
            conn.close()

        return ret

    @classmethod
    def load_from_pickel(cls, base_dir, id):
        return pickle.load(open("{}/{}.pkl".format(base_dir, id), "rb"))

    def __init__(self, id, author, pub_time, url, topics):

        self.id = id
        self.author = author
        self.pub_time = pub_time
        self.url = url
        # Topics is a list of tuple of topic category and probability.
        # e.g. [(1, 0.12), (2, 0.14), (3, 0.98),...]
        self.topics = topics

    def get_topic_prob(self, t_id):

        '''
        Get probability of given topic id
        :param t_id:
        :return: probability
        '''
        for t in self.topics:
            if t[0] == t_id:
                return t[1]

        return None
